"""
Simplified python code documentation
"""
from .doc import append_doc, append_plot,append_str, insert_doc,insert_str, insert_eq,table
__all__ = ['append_doc', 'append_plot', 'append_str','insert_doc', 'insert_str','insert_eq', 'table']